@@ -67,7 +67,7 @@ ags_wave_export_dialog_file_open_button_callback(GtkWidget *file_open_button,
   sandbox_path = NULL;
 
 #if defined(AGS_MACOS_SANDBOX)
-  sandbox_path = g_strdup_printf("%s/Library/%s",
+  sandbox_path = g_strdup_printf("%s/Library/%s/Data",
 				 home_path,
 				 AGS_DEFAULT_BUNDLE_ID);
 
